/*
 *	codes from COSMOS system
 *		Mar.1991	Yutaka Satou <ysato@etl.go.jp>
 */
char *getenv();
#include <ctype.h>
#include <stdio.h>
#include <errno.h>
extern int errno;
#include <fcntl.h>

char *Onew_env_substitute(istr,ostr)
	char *istr,*ostr;
{	register char *sp,*tp,*np,*op;
	char name[1000],*body;
	char *rstr = istr;

	op = ostr;
	for( sp = istr; *sp; sp++ ){
		if( *sp == '$' ){
			np = name;
			for( tp = sp+1; *tp; tp++ ){
				if( !isalnum(*tp) && *tp != '_' )
					break;
				*np++ = *tp;
			}
			*np = 0;
			if( body = getenv(name) ){
				rstr = ostr;
				strcpy(op,body);
				op += strlen(op);
				sp = tp;
			}
		}
		*op++ = *sp;
	}
	*op = 0;
	if( rstr == ostr ){
		char buf[10000];
		strcpy(buf,ostr);
		Onew_env_substitute(buf,ostr);
	}
	return rstr;
}

Onew_searchPath(file,name,path)
	char *file,*name,*path;
{	char xpath[10000],*sp,*dp;
	int fd;

	Onew_env_substitute(path,xpath);
	sp = xpath;

	while( *sp ){
		dp = file;
		for( ; *sp && *sp != ':'; sp++ )
			*dp++ = *sp;
		*dp++ = '/';
		strcpy(dp,name);
		if( 0 <= (fd = open(file,0)) )
			return fd;

		if( *sp == ':' )
			sp++;
		else	break;
	}
	return -1;
}


#ifdef HAS_SELECT	/* normally 4.xBSD is the case */
#include <sys/time.h>
Onew_fd_input_ready( fd, wait_usec )
{	struct timeval tv;
	int mask,nready;

	if( wait_usec ){
		tv.tv_sec  = wait_usec / 1000000;
		tv.tv_usec = wait_usec % 1000000;
	}
	mask = 1 << fd;

	nready = select(32,&mask,0,0, wait_usec ? &tv : 0);
	if( nready < 0 ){
		if( errno != EINTR ){
			Onew_message(" ERROR! select(%d,0x%x) TIMEOUT=%dusec",
				fd,mask,wait_usec);
		}
		return -1; /* SELECT INTERRUPTED */
	}
	return mask ? 1 : 0;
}

#endif
#ifdef HAS_POLL		/* normally SVR[34] is the case */
/*
 * by Jun Ohta
 */
#include <poll.h>
Onew_fd_input_ready( fd, wait_usec )
{	struct pollfd pfd;
	int nready;

	pfd.fd = fd;
	pfd.events = POLLIN;
	nready = poll(&pfd, 1L, wait_usec? (wait_usec / 1000): -1);
	if( nready < 0 ){
		Onew_message(" ERROR! poll(%d) TIMEOUT=%dusec",pfd,wait_usec);
		return -1; /* POLL INTERRUPTED */
	}
	return (pfd.revents == POLLIN)? 1: 0;
}
#endif

#ifdef __linux__
#define READYCC(fp)	(fp->_egptr - fp->_gptr)
#else
#define READYCC(fp)	(fp->_cnt)
#endif

Onew_fp_input_ready( fp, wait_usec )
	FILE *fp;
{
	if( fp && READYCC(fp) )
		return 1;

	if( wait_usec < 0 )
		return 0;

	return Onew_fd_input_ready(fileno(fp), wait_usec);
}

Onew_msleep(msec)
{
/*
	usleep(msec*1000);
*/
	sleep(msec/1000);
}

static int (*message)();
Onew_message(fmt,a,b,c,d,e,f,g)
{
	if( message )
		(*message)(fmt,a,b,c,d,e,f,g);
	else	fprintf(stderr,fmt,a,b,c,d,e,f,g);
}
onew_messagefunc(func)
	int (*func)();
{
	message = func;
}


#ifdef SYS_BSD
#include <sgtty.h>
#endif
#ifdef SYS_SYSV
#include <sys/termio.h>
#endif
#ifdef SYS_POSIX
#include <sys/termios.h>
#endif

#ifdef hpux
#include <sys/file.h>
#define	TIOCNOTTY	O_NOCTTY
#endif

Onew_notty()
{	int fd;

	if( 0 < (fd = open("/dev/tty")) ){
		ioctl(fd,TIOCNOTTY,0);
		close(fd);
	}
}


#ifdef SYS_BSD
Onew_ttyIcanon(on)
{	int fd;
	struct sgttyb buf;

	fd = open("/dev/tty", O_RDWR);
	ioctl(fd, TIOCGETP, &buf);
	if( on )
		buf.sg_flags |= ECHO, buf.sg_flags &= ~CBREAK;
	else	buf.sg_flags &= ~ECHO, buf.sg_flags |= CBREAK;
	ioctl(fd, TIOCSETN, &buf);
	close(fd);
}
#endif
#ifdef SYS_SYSV
Onew_ttyIcanon(on)
{	int fd;
	struct termio buf;

	fd = open("/dev/tty", O_RDWR);
	ioctl(fd, TCGETA, &buf);
	if( on )
		buf.c_lflag |= (ECHO|ICANON);
	else	buf.c_lflag &= ~(ECHO|ICANON);
	ioctl(fd, TCSETAW, &buf);
	close(fd);
}
#endif
#ifdef SYS_POSIX
Onew_ttyIcanon(on)
{	int fd;
	struct termios buf;

	fd = open("/dev/tty", O_RDWR);
	tcgetattr(fd, &buf);
	if( on )
		buf.c_lflag |= (ECHO|ICANON);
	else	buf.c_lflag &= ~(ECHO|ICANON);
	tcsetattr(fd, TCSADRAIN, &buf);
	close(fd);
}
#endif


static FILE *LOG;
Onew_dbglog(fmt,a,b,c,d,e,f,g)
{	char *logfile;

	if( LOG == (FILE*)-1 )
		return;
	if( LOG == 0 ){
		if((logfile = getenv("ONEW_DBGLOG")) == 0
		|| (LOG = fopen(logfile,"w")) == NULL){
			LOG = (FILE*)-1;
			return;
		}
	}
	fprintf(LOG,fmt,a,b,c,d,e,f,g);
	fflush(LOG);
}


Onew_non_ascii(str)
	char *str;
{	char *s;

	for(s = str; *s; s++)
		if( *s & 0x80 )
			return 1;
	return 0;
}
